Package: gmodels
Version: 2.19.1
Date: 2024-03-05
Title: Various R Programming Tools for Model Fitting
URL: https://github.com/r-gregmisc/gmodels
BugReports: https://github.com/r-gregmisc/gmodels/issues
Authors@R: c(
  person(c('Gregory', 'R.'), 'Warnes', email='greg@warnes.net', role=c('aut', 'cre')),
  person('Ben', 'Bolker', role='aut'),
  person('Thomas', 'Lumley', role='aut'),
  person(c('Randall', 'C.'), 'Johnson', role='aut', comment='Contributions from Randall C. Johnson are Copyright (2005) SAIC-Frederick, Inc. Funded by the Intramural Research
        Program, of the NIH, National Cancer Institute, Center for Cancer Research under NCI Contract NO1-CO-12400'),
  person('Airel', 'Muldoon', role='ctb',  email='Ariel.Muldoon@oregonstate.edu'),
  person('Nitin', 'Jain', role='aut'),
  person('Dirk', 'Enzmann', role='ctb', email='dirk.enzmann@jura.uni-hamburg.de'),
  person('Søren', 'Højsgaards', role='ctb', email='sorenh@agrsci.dk'),
  person('Ulrich', 'Halekoh', role='ctb', email='ulrich.halekoh@agrsci.dk'),
  person('Mark', 'Schwartz', role='aut'),
  person('Jim', 'Rogers', role='aut')
  )
Maintainer: Gregory R. Warnes <greg@warnes.net>
Description: Various R programming tools for model fitting.
Suggests: gplots, gtools, Matrix, nlme, lme4
Imports: MASS, gdata, stats
License: GPL-2
NeedsCompilation: no
RoxygenNote: 7.3.1
Encoding: UTF-8
Packaged: 2024-03-06 15:32:38 UTC; warnes
Author: Gregory R. Warnes [aut, cre],
  Ben Bolker [aut],
  Thomas Lumley [aut],
  Randall C. Johnson [aut] (Contributions from Randall C. Johnson are
    Copyright (2005) SAIC-Frederick, Inc. Funded by the Intramural
    Research Program, of the NIH, National Cancer Institute, Center for
    Cancer Research under NCI Contract NO1-CO-12400),
  Airel Muldoon [ctb],
  Nitin Jain [aut],
  Dirk Enzmann [ctb],
  Søren Højsgaards [ctb],
  Ulrich Halekoh [ctb],
  Mark Schwartz [aut],
  Jim Rogers [aut]
Repository: CRAN
Date/Publication: 2024-03-06 16:00:02 UTC
