# Generated by roxygen2: do not edit by hand

S3method(base::print,fit_contrast)
S3method(base::print,glh.test)
S3method(base::summary,glh.test)
S3method(gmodels::ci,binom)
S3method(gmodels::ci,estimable)
S3method(gmodels::ci,fit_contrast)
S3method(gmodels::ci,lm)
S3method(gmodels::ci,lme)
S3method(gmodels::ci,numeric)
S3method(gmodels::est_p_ci,fit_contrast)
S3method(gmodels::est_p_ci,lm)
S3method(gmodels::estimable,default)
S3method(gmodels::estimable,mlm)
S3method(gmodels::fit.contrast,lm)
S3method(gmodels::fit.contrast,lme)
S3method(stats::coef,fit_contrast)
export(.to.est)
export(CrossTable)
export(ci)
export(ci.binom)
export(coefFrame)
export(est_p_ci)
export(estimable)
export(fast.prcomp)
export(fast.svd)
export(fit.contrast)
export(glh.test)
export(make.contrasts)
importFrom(MASS,ginv)
importFrom(gdata,frameApply)
importFrom(gdata,nobs)
importFrom(stats,chisq.test)
importFrom(stats,coef)
importFrom(stats,family)
importFrom(stats,fisher.test)
importFrom(stats,mcnemar.test)
importFrom(stats,pchisq)
importFrom(stats,pf)
importFrom(stats,pt)
importFrom(stats,qbeta)
importFrom(stats,qt)
importFrom(stats,sd)
importFrom(stats,summary.glm)
importFrom(stats,summary.lm)
